@@ -1,15 +1,14 @@
 # This script activates the currently inactive generation.
 
-from optparse import make_option
-from django.core.management.base import NoArgsCommand
+from django.core.management.base import BaseCommand
 from mapit.models import Generation
 
 
-class Command(NoArgsCommand):
+class Command(BaseCommand):
     help = 'Actives the inactive generation'
-    option_list = NoArgsCommand.option_list + (
-        make_option('--commit', action='store_true', dest='commit', help='Actually update the database'),
-    )
+
+    def add_arguments(self, parser):
+        parser.add_argument('--commit', action='store_true', dest='commit', help='Actually update the database')
 
     def handle(self, **options):
         new = Generation.objects.new()
