@@ -1,23 +1,22 @@
 # This script deactivates a particular generation
 
-from optparse import make_option
 from django.core.management.base import BaseCommand, CommandError
 from mapit.models import Generation
 
 
 class Command(BaseCommand):
     help = 'Deactivate a generation'
-    args = '<GENERATION-ID>'
-    option_list = BaseCommand.option_list + (
-        make_option('--commit', action='store_true', dest='commit',
-                    help='Actually update the database'),
-        make_option('--force', action='store_true', dest='force',
-                    help='Force deactivation, even if it would leave no active generations'))
 
-    def handle(self, generation_id, **options):
-        generation_to_deactivate = Generation.objects.get(id=int(generation_id, 10))
+    def add_arguments(self, parser):
+        parser.add_argument('generation_id', type=int)
+        parser.add_argument('--commit', action='store_true', dest='commit', help='Actually update the database')
+        parser.add_argument('--force', action='store_true', dest='force',
+                            help='Force deactivation, even if it would leave no active generations')
+
+    def handle(self, **options):
+        generation_to_deactivate = Generation.objects.get(id=options['generation_id'])
         if not generation_to_deactivate.active:
-            raise CommandError("The generation %s wasn't active" % (generation_id,))
+            raise CommandError("The generation %d wasn't active" % (options['generation_id'],))
         active_generations = Generation.objects.filter(active=True).count()
         if active_generations <= 1 and not options['force']:
             raise CommandError(
