@@ -6,11 +6,11 @@
 # the GPL.  Based on import_norway_osm.py by Matthew Somerville
 
 import csv
-from optparse import make_option
 
 from django.core.management.base import LabelCommand
 from django.contrib.gis.geos import GEOSGeometry
-from django.utils.six import Iterator
+from django.contrib.gis.db.models import Union
+from six import Iterator
 
 from mapit.models import Area, Generation, Geometry, Country, Type
 from mapit.management.command_utils import save_polygons
@@ -38,10 +38,11 @@ class CommentedFile(Iterator):
 
 class Command(LabelCommand):
     help = 'Import region data'
-    args = '<CSV file listing name and which existing areas to combine into regions>'
-    option_list = LabelCommand.option_list + (
-        make_option('--commit', action='store_true', dest='commit', help='Actually update the database'),
-    )
+    label = '<CSV file listing name and which existing areas to combine into regions>'
+
+    def add_arguments(self, parser):
+        super(Command, self).add_arguments(parser)
+        parser.add_argument('--commit', action='store_true', dest='commit', help='Actually update the database')
 
     def handle_label(self, filename, **options):
         current_generation = Generation.objects.current()
@@ -100,7 +101,7 @@ class Command(LabelCommand):
                             geometry = Geometry.objects.filter(**args)
                     except:
                         raise Exception("Area or geometry with name %s was not found!" % name)
-                    unionoutline = geometry.unionagg()
+                    unionoutline = geometry.aggregate(Union('polygon'))['polygon__union']
 
                 def update_or_create():
                     try:
