@@ -5,7 +5,7 @@
 # the command line
 
 import csv
-from optparse import make_option
+from django.db import transaction
 from django.contrib.gis.geos import Point
 from django.core.management.base import LabelCommand
 from django.conf import settings
@@ -14,69 +14,70 @@ from mapit.models import Postcode
 
 class Command(LabelCommand):
     help = 'Import Postal codes from a CSV file or files'
-    args = '<CSV files>'
+    label = '<CSV file>'
     count = {'total': 0, 'updated': 0, 'unchanged': 0, 'created': 0}
     often = 1000
 
     option_defaults = {}
-    option_list = LabelCommand.option_list + (
-        make_option(
+
+    def add_arguments(self, parser):
+        super(Command, self).add_arguments(parser)
+        parser.add_argument(
             '--code-field',
             action='store',
             dest='code-field',
             default=1,
             help='The column of the CSV containing the postal code (default 1, first)'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--coord-field-lat',
             action='store',
             dest='coord-field-lat',
             default=2,
             help='The column of the CSV containing the lat/y co-ordinate (default 2)'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--coord-field-lon',
             action='store',
             dest='coord-field-lon',
             default=None,
             help='The column of the CSV containing the lon/x co-ordinate (default --coord-field-lat + 1)'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--header-row',
             action='store_true',
             dest='header-row',
             default=False,
             help='Set if the CSV file has a header row'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--no-location',
             action="store_false",
             dest='location',
             default=True,
             help='Set if the postal codes have no associated location (still useful for existence checks)'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--srid',
             action="store",
             dest='srid',
             default=4326,
             help='The SRID of the projection for the data given (default 4326 WGS-84)'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--strip',
             action="store_true",
             dest='strip',
             default=False,
             help='Whether to strip all spaces from the postal code before import'
-        ),
-        make_option(
+        )
+        parser.add_argument(
             '--tabs',
             action="store_true",
             dest='tabs',
             default=False,
             help='If the CSV file actually uses tab as its separator'
-        ),
-    )
+        )
 
     def handle_label(self, file, **options):
         self.process(file, options)
@@ -90,14 +91,17 @@ class Command(LabelCommand):
         if options['header-row']:
             next(reader)
         for row in reader:
-            self.code = row[int(options['code-field']) - 1].strip()
-            if options['strip']:
-                self.code = self.code.replace(' ', '')
-            if not self.pre_row(row, options):
-                continue
+            self._process_row(row, options)
+        self.print_stats()
+
+    @transaction.atomic
+    def _process_row(self, row, options):
+        self.code = row[int(options['code-field']) - 1].strip()
+        if options['strip']:
+            self.code = self.code.replace(' ', '')
+        if self.pre_row(row, options):
             pc = self.handle_row(row, options)
             self.post_row(pc)
-        self.print_stats()
 
     def pre_row(self, row, options):
         return True
@@ -105,8 +109,11 @@ class Command(LabelCommand):
     def post_row(self, pc):
         return True
 
+    def location_available_for_row(self, row):
+        return True
+
     def handle_row(self, row, options):
-        if not options['location']:
+        if not options['location'] or not self.location_available_for_row(row):
             return self.do_postcode()
 
         if not options['coord-field-lon']:
@@ -123,23 +130,23 @@ class Command(LabelCommand):
         try:
             pc = Postcode.objects.get(postcode=self.code)
             if location:
-                curr_location = (pc.location[0], pc.location[1])
-                if settings.MAPIT_COUNTRY == 'GB':
-                    if pc.postcode[0:2] == 'BT':
-                        curr_location = pc.as_irish_grid()
+                if pc.location:
+                    curr_location = pc.location
+                    # Postcode locations are stored as WGS84
+                    if settings.MAPIT_COUNTRY == 'GB':
+                        curr_location = pc.as_uk_grid()
+                    elif srid != 4326:
+                        curr_location = pc.location.transform(srid, clone=True)
+                    if curr_location[0] != location[0] or curr_location[1] != location[1]:
+                        pc.location = location
+                        pc.save()
+                        self.count['updated'] += 1
                     else:
-                        pc.location.transform(27700)  # Postcode locations are stored as WGS84
-                        curr_location = (pc.location[0], pc.location[1])
-                    curr_location = tuple(map(round, curr_location))
-                elif srid != 4326:
-                    pc.location.transform(srid)  # Postcode locations are stored as WGS84
-                    curr_location = (pc.location[0], pc.location[1])
-                if curr_location[0] != location[0] or curr_location[1] != location[1]:
+                        self.count['unchanged'] += 1
+                else:
                     pc.location = location
                     pc.save()
                     self.count['updated'] += 1
-                else:
-                    self.count['unchanged'] += 1
             else:
                 self.count['unchanged'] += 1
         except Postcode.DoesNotExist:
