@@ -2,7 +2,6 @@
 # and output it as either HTML or JSON appropriately
 
 from django import http
-from django.template import RequestContext
 from django.template.loader import render_to_string
 from mapit.shortcuts import output_json
 
@@ -12,6 +11,13 @@ class ViewException(Exception):
 
 
 class ViewExceptionMiddleware(object):
+    def __init__(self, get_response):
+        self.get_response = get_response
+
+    def __call__(self, request):
+        response = self.get_response(request)
+        return response
+
     def process_exception(self, request, exception):
         if not isinstance(exception, ViewException):
             return None
@@ -27,6 +33,6 @@ class ViewExceptionMiddleware(object):
             return response_type(render_to_string(
                 'mapit/%s.html' % code,
                 {'error': message},
-                context_instance=RequestContext(request)
+                request=request
             ))
         return output_json({'error': message}, code=code)
