@@ -1,21 +1,21 @@
+from __future__ import unicode_literals
+
 import re
 import itertools
 
+from six import python_2_unicode_compatible
 from django.contrib.gis.db import models
-from django.contrib.gis.gdal import SRSException, OGRException
 from django.conf import settings
 from django.db import connection
 from django.db.models.query import RawQuerySet
-from django.utils.html import escape
-from django.utils.encoding import python_2_unicode_compatible
+from django.utils.encoding import smart_text
+from django.utils.functional import cached_property
 
-from mapit.managers import Manager, GeoManager
 from mapit import countries
-from mapit.djangopatch import GetQuerySetMetaclass
-from django.utils import six
+from mapit.geometryserialiser import GeometrySerialiser
 
 
-class GenerationManager(six.with_metaclass(GetQuerySetMetaclass, models.Manager)):
+class GenerationManager(models.Manager):
     def current(self):
         """Return the most recent active generation.
 
@@ -83,6 +83,9 @@ class Generation(models.Model):
 
     objects = GenerationManager()
 
+    class Meta:
+        ordering = ('id',)
+
     def __str__(self):
         id = self.id or '?'
         return "Generation %s (%sactive)" % (id, "" if self.active else "in")
@@ -126,9 +129,10 @@ class Type(models.Model):
         return '%s (%s)' % (self.description, self.code)
 
 
-class AreaManager(six.with_metaclass(GetQuerySetMetaclass, models.GeoManager)):
+class AreaManager(models.Manager):
     def get_queryset(self):
-        return super(AreaManager, self).get_queryset().select_related('type', 'country')
+        return super(AreaManager, self).get_queryset().select_related(
+            'type', 'country', 'parent_area').prefetch_related('countries')
 
     def by_location(self, location, generation=None):
         if generation is None:
@@ -219,32 +223,24 @@ SELECT DISTINCT mapit_area.*
         return area
 
 
-class TransformError(Exception):
-    pass
-
-
 @python_2_unicode_compatible
 class Area(models.Model):
     name = models.CharField(max_length=2000, blank=True)
-    parent_area = models.ForeignKey('self', related_name='children', null=True, blank=True)
-    type = models.ForeignKey(Type, related_name='areas')
-    country = models.ForeignKey(Country, related_name='areas', null=True, blank=True)
-    generation_low = models.ForeignKey(Generation, related_name='new_areas', null=True)
-    generation_high = models.ForeignKey(Generation, related_name='final_areas', null=True)
+    parent_area = models.ForeignKey('self', related_name='children', null=True, blank=True, on_delete=models.CASCADE)
+    type = models.ForeignKey(Type, related_name='areas', on_delete=models.CASCADE)
+    country = models.ForeignKey(Country, related_name='areas_direct', null=True, blank=True, on_delete=models.CASCADE)
+    countries = models.ManyToManyField(Country, related_name='areas_m2m', blank=True)
+    generation_low = models.ForeignKey(Generation, related_name='new_areas', null=True, on_delete=models.CASCADE)
+    generation_high = models.ForeignKey(Generation, related_name='final_areas', null=True, on_delete=models.CASCADE)
 
     objects = AreaManager()
 
     class Meta:
         ordering = ('name', 'type')
 
-    @property
+    @cached_property
     def all_codes(self):
-        if not getattr(self, 'code_list', None):
-            self.code_list = self.codes.select_related('type')
-        codes = {}
-        for code in self.code_list:
-            codes[code.type.code] = code.code
-        return codes
+        return {code.type.code: code.code for code in self.codes.select_related('type')}
 
     def __str__(self):
         name = self.name or '(unknown)'
@@ -252,7 +248,7 @@ class Area(models.Model):
 
     def as_dict(self, all_names=None):
         all_names = all_names or []
-        return {
+        out = {
             'id': self.id,
             'name': self.name,
             'parent_area': self.parent_area_id,
@@ -265,20 +261,20 @@ class Area(models.Model):
             'codes': self.all_codes,
             'all_names': dict(n.as_tuple() for n in all_names),
         }
+        countries = self.all_m2m_countries
+        if countries:
+            out['countries'] = countries
+        return out
 
-    def css_indent_class(self):
-        """Get a CSS class for use on <li> representations of this area
+    def list_countries(self):
+        countries = [c['name'] for c in self.all_m2m_countries]
+        if self.country:
+            countries.append(self.country.name)
+        return countries
 
-        Currently this is only used to indicate the indentation level
-        that should be used on the code types O02, O03, O04 ... O011,
-        which are only used by global MapIt.
-        """
-        m = re.search(r'^O([01][0-9])$', self.type.code)
-        if m:
-            level = int(m.group(1), 10)
-            return "area_level_%d" % (level,)
-        else:
-            return ""
+    @cached_property
+    def all_m2m_countries(self):
+        return [{'code': c.code, 'name': c.name} for c in self.countries.all()]
 
     def export(self,
                srid,
@@ -311,73 +307,29 @@ class Area(models.Model):
         something else goes wrong with the spatial transform, then a
         TransformError exception is raised.
         """
-        all_areas = self.polygons.all()
-        if len(all_areas) > 1:
-            all_areas = all_areas.collect()
-        elif len(all_areas) == 1:
-            all_areas = all_areas[0].polygon
-        else:
+        all_polygons = self.polygons.all()
+        if len(all_polygons) == 0:
             return (None, None)
-
-        if srid != settings.MAPIT_AREA_SRID:
-            try:
-                all_areas.transform(srid)
-            except (SRSException, OGRException) as e:
-                raise TransformError("Error with transform: %s" % e)
-
-        num_points_before_simplification = all_areas.num_points
-        if simplify_tolerance:
-            all_areas = all_areas.simplify(simplify_tolerance)
-            if all_areas.num_points == 0 and num_points_before_simplification > 0:
-                raise TransformError("Simplifying %s with tolerance %f left no boundary at all" % (
-                    self, simplify_tolerance))
-
+        serialiser = GeometrySerialiser(self, srid, simplify_tolerance)
         if export_format == 'kml':
-            if kml_type == "polygon":
-                out = all_areas.kml
-            elif kml_type == "full":
-                out = '''<?xml version="1.0" encoding="UTF-8"?>
-<kml xmlns="http://www.opengis.net/kml/2.2">
-    <Document>
-        <Style id="ourPolygonStyle">
-            <LineStyle>
-                <color>%s</color>
-                <width>2</width>
-            </LineStyle>
-            <PolyStyle>
-                <color>%s</color>
-            </PolyStyle>
-        </Style>
-        <Placemark>
-            <styleUrl>#ourPolygonStyle</styleUrl>
-            <name>%s</name>
-            %s
-        </Placemark>
-    </Document>
-</kml>''' % (line_colour, fill_colour, escape(self.name), all_areas.kml)
-            else:
-                raise Exception("Unknown kml_type: '%s'" % (kml_type,))
-            content_type = 'application/vnd.google-earth.kml+xml'
+            out, content_type = serialiser.kml(kml_type, line_colour, fill_colour)
         elif export_format in ('json', 'geojson'):
-            out = all_areas.json
-            content_type = 'application/json'
+            out, content_type = serialiser.geojson()
         elif export_format == 'wkt':
-            out = all_areas.wkt
-            content_type = 'text/plain'
+            out, content_type = serialiser.wkt()
         return (out, content_type)
 
 
 @python_2_unicode_compatible
 class Geometry(models.Model):
-    area = models.ForeignKey(Area, related_name='polygons')
+    area = models.ForeignKey(Area, related_name='polygons', on_delete=models.CASCADE)
     polygon = models.PolygonField(srid=settings.MAPIT_AREA_SRID)
-    objects = GeoManager()
 
     class Meta:
         verbose_name_plural = 'geometries'
 
     def __str__(self):
-        return '%s, polygon %d' % (self.area, self.id)
+        return '%s, polygon %d' % (smart_text(self.area), self.id)
 
 
 @python_2_unicode_compatible
@@ -390,10 +342,10 @@ class NameType(models.Model):
     # and displayed in the alternative names section.
 
     code = models.CharField(
-        max_length=10, unique=True, help_text="A unique code to identify this type of name: eg 'english' or 'iso'")
+        max_length=500, unique=True, help_text="A unique code to identify this type of name: eg 'english' or 'iso'")
     description = models.CharField(
         max_length=200, blank=True, help_text="The name of this type of name, eg 'English' or 'ISO Standard'")
-    objects = Manager()
+    objects = models.Manager()
 
     def __str__(self):
         return '%s (%s)' % (self.description, self.code)
@@ -401,10 +353,10 @@ class NameType(models.Model):
 
 @python_2_unicode_compatible
 class Name(models.Model):
-    area = models.ForeignKey(Area, related_name='names')
-    type = models.ForeignKey(NameType, related_name='names')
+    area = models.ForeignKey(Area, related_name='names', on_delete=models.CASCADE)
+    type = models.ForeignKey(NameType, related_name='names', on_delete=models.CASCADE)
     name = models.CharField(max_length=2000)
-    objects = Manager()
+    objects = models.Manager()
 
     class Meta:
         unique_together = ('area', 'type')
@@ -430,7 +382,7 @@ class CodeType(models.Model):
     # This could be extended to a more generic data store of information on an
     # object, perhaps.
 
-    code = models.CharField(max_length=10, unique=True, help_text="A unique code, eg 'ons' or 'unit_id'")
+    code = models.CharField(max_length=500, unique=True, help_text="A unique code, eg 'ons' or 'unit_id'")
     description = models.CharField(
         max_length=200, blank=True,
         help_text="The name of the code, eg 'Office of National Statitics' or 'Ordnance Survey ID'")
@@ -441,10 +393,10 @@ class CodeType(models.Model):
 
 @python_2_unicode_compatible
 class Code(models.Model):
-    area = models.ForeignKey(Area, related_name='codes')
-    type = models.ForeignKey(CodeType, related_name='codes')
+    area = models.ForeignKey(Area, related_name='codes', on_delete=models.CASCADE)
+    type = models.ForeignKey(CodeType, related_name='codes', on_delete=models.CASCADE)
     code = models.CharField(max_length=500)
-    objects = Manager()
+    objects = models.Manager()
 
     class Meta:
         unique_together = ('area', 'type')
@@ -455,12 +407,24 @@ class Code(models.Model):
 
 # Postcodes
 
-class PostcodeManager(six.with_metaclass(GetQuerySetMetaclass, GeoManager)):
-    def get_queryset(self):
-        return self.model.QuerySet(self.model)
+class PostcodeQuerySet(models.QuerySet):
+    # ST_CoveredBy on its own does not appear to use the index.
+    # Plus this way we can keep the polygons in the database
+    # without pulling out in a giant WKB string
+    def filter_by_area(self, area):
+        collect = '''ST_Transform((select ST_Collect(polygon) from mapit_geometry
+            where area_id=%s group by area_id), 4326)'''
+        return self.extra(
+            where=[
+                'location && %s' % collect,
+                'ST_CoveredBy(location, %s)' % collect
+            ],
+            params=[area.id, area.id]
+        )
 
-    def __getattr__(self, attr, *args):
-        return getattr(self.get_queryset(), attr, *args)
+
+def str2int(s):
+    return int(round(float(s)))
 
 
 @python_2_unicode_compatible
@@ -470,26 +434,11 @@ class Postcode(models.Model):
     # Will hopefully use PostGIS point-in-polygon tests, but if we don't have the polygons...
     areas = models.ManyToManyField(Area, related_name='postcodes', blank=True)
 
-    objects = PostcodeManager()
+    objects = PostcodeQuerySet.as_manager()
 
     class Meta:
         ordering = ('postcode',)
 
-    class QuerySet(models.query.GeoQuerySet):
-        # ST_CoveredBy on its own does not appear to use the index.
-        # Plus this way we can keep the polygons in the database
-        # without pulling out in a giant WKB string
-        def filter_by_area(self, area):
-            collect = '''ST_Transform((select ST_Collect(polygon) from mapit_geometry
-                where area_id=%s group by area_id), 4326)'''
-            return self.extra(
-                where=[
-                    'location && %s' % collect,
-                    'ST_CoveredBy(location, %s)' % collect
-                ],
-                params=[area.id, area.id]
-            )
-
     def __str__(self):
         return self.get_postcode_display()
 
@@ -514,12 +463,13 @@ class Postcode(models.Model):
             countries.augment_postcode(self, result)
         return result
 
-    # Doing this via self.location.transform(29902) gives incorrect results.
-    # The database has the right proj4 text, the proj file does not. I think.
-    def as_irish_grid(self):
+    # Doing this via self.location.transform(27700/29902) can give incorrect results
+    # with some versions of GDAL. Via the database produces a correct result.
+    def as_uk_grid(self):
         cursor = connection.cursor()
-        cursor.execute("SELECT ST_AsText(ST_Transform(ST_GeomFromText('POINT(%f %f)', 4326), 29902))" % (
-            self.location[0], self.location[1]))
+        srid = 29902 if self.postcode[0:2] == 'BT' else 27700
+        cursor.execute("SELECT ST_AsText(ST_Transform(ST_GeomFromText('POINT(%f %f)', 4326), %d))" % (
+            self.location[0], self.location[1], srid))
         row = cursor.fetchone()
-        m = re.match('POINT\((.*?) (.*)\)', row[0])
-        return list(map(float, m.groups()))
+        m = re.match(r'POINT\((.*?) (.*)\)', row[0])
+        return list(map(str2int, m.groups()))
