@@ -1,24 +1,17 @@
-import json
-import django
+import itertools
+import re
+
+from six.moves import map
 from django import http
 from django.db import connection
 from django.conf import settings
 from django.shortcuts import get_object_or_404 as orig_get_object_or_404
-from django.shortcuts import render_to_response
-from django.template import RequestContext
+from django.template import loader
+from django.utils.translation import ugettext as _
+
+from mapit.iterables import defaultiter
 
 from django.core.serializers.json import DjangoJSONEncoder
-# Assuming at least python 2.6, in Django < 1.6, the above class is either a
-# packaged simplejson subclass if simplejson is installed, or a core json
-# subclass. In Django >= 1.6, it is always a core json subclass. The json.dump
-# call in this file needs to be the same thing that the above class is.
-if django.get_version() < '1.6':
-    try:
-        import simplejson
-        if issubclass(DjangoJSONEncoder, simplejson.JSONEncoder):
-            import simplejson as json  # noqa
-    except:
-        pass
 
 
 class GEOS_JSONEncoder(DjangoJSONEncoder):
@@ -30,42 +23,25 @@ class GEOS_JSONEncoder(DjangoJSONEncoder):
         return super(GEOS_JSONEncoder, self).default(o)
 
 
-def render(request, template_name, context=None):
-    if context is None:
-        context = {}
-#    context['base'] = base or 'base.html'
-#    context['connection'] = connection
-    return render_to_response(
-        template_name, context, context_instance=RequestContext(request)
-    )
-
-
-def sorted_areas(areas):
-    # In here to prevent a circular dependency
-    from mapit import countries
-    if hasattr(countries, 'sorted_areas'):
-        return countries.sorted_areas(areas)
-    return list(areas)
-
-
 def output_html(request, title, areas, **kwargs):
-    kwargs['json_url'] = request.get_full_path().replace('.html', '')
+    kwargs['json_url'] = re.sub(r'(\.map)?\.html', '', request.get_full_path())
     kwargs['title'] = title
-    kwargs['areas'] = sorted_areas(areas)
-    kwargs['indent_areas'] = kwargs.get('indent_areas', False)
-    return render(request, 'mapit/data.html', kwargs)
+    tpl = loader.render_to_string('mapit/data.html', kwargs, request=request)
+    wraps = tpl.split('!!!DATA!!!')
+
+    indent_areas = kwargs.get('indent_areas', False)
+    show_map = kwargs.get('show_map', False)
+    item_tpl = loader.get_template('mapit/areas_item.html')
+    areas = map(lambda area: item_tpl.render({
+        'area': area, 'indent_areas': indent_areas, 'show_map': show_map,
+    }), areas)
+    areas = defaultiter(areas, '<li>' + _('No matching areas found.') + '</li>')
+    content = itertools.chain(wraps[0:1], areas, wraps[1:])
+
+    return http.StreamingHttpResponse(content)
 
 
 def output_json(out, code=200):
-    types = {
-        400: http.HttpResponseBadRequest,
-        404: http.HttpResponseNotFound,
-        500: http.HttpResponseServerError,
-    }
-    response_type = types.get(code, http.HttpResponse)
-    response = response_type(content_type='application/json; charset=utf-8')
-    response['Access-Control-Allow-Origin'] = '*'
-    response['Cache-Control'] = 'max-age=2419200'  # 4 weeks
     if code != 200:
         out['code'] = code
     indent = None
@@ -73,7 +49,34 @@ def output_json(out, code=200):
         if isinstance(out, dict):
             out['debug_db_queries'] = connection.queries
         indent = 4
-    json.dump(out, response, ensure_ascii=False, cls=GEOS_JSONEncoder, indent=indent)
+    encoder = GEOS_JSONEncoder(ensure_ascii=False, indent=indent)
+    content = encoder.iterencode(out)
+
+    # We don't want a generator function (iterencode) to be passed to an
+    # HttpResponse, as it won't cache due to its close() function adding it to
+    # an instance attribute.
+    content = map(lambda x: x, content)
+
+    types = {
+        400: http.HttpResponseBadRequest,
+        404: http.HttpResponseNotFound,
+        500: http.HttpResponseServerError,
+    }
+    response_type = types.get(code, http.StreamingHttpResponse)
+
+    response = response_type(content_type='application/json; charset=utf-8')
+    response['Access-Control-Allow-Origin'] = '*'
+    response['Cache-Control'] = 'max-age=2419200'  # 4 weeks
+    attr = 'streaming_content' if getattr(response, 'streaming', None) else 'content'
+    setattr(response, attr, content)
+    return response
+
+
+def output_polygon(content_type, output):
+    response = http.HttpResponse(content_type='%s; charset=utf-8' % content_type)
+    response['Access-Control-Allow-Origin'] = '*'
+    response['Cache-Control'] = 'max-age=2419200'  # 4 weeks
+    response.write(output)
     return response
 
 
