@@ -3,8 +3,11 @@ import re
 from mapit import countries
 
 
+re_number = r'[+-]?(?:\d*\.)?\d+'
+
+
 def is_valid_postcode(pc):
-    pc = re.sub('\s+', '', pc.upper())
+    pc = re.sub(r'\s+', '', pc.upper())
 
     if hasattr(countries, 'is_valid_postcode'):
         return countries.is_valid_postcode(pc)
@@ -12,7 +15,7 @@ def is_valid_postcode(pc):
 
 
 def is_valid_partial_postcode(pc):
-    pc = re.sub('\s+', '', pc.upper())
+    pc = re.sub(r'\s+', '', pc.upper())
 
     if hasattr(countries, 'is_valid_partial_postcode'):
         return countries.is_valid_partial_postcode(pc)
