@@ -1,18 +1,22 @@
+from operator import attrgetter
 import re
 import itertools
-from psycopg2.extensions import QueryCanceledError
-try:
-    from django.db.utils import DatabaseError
-except:
-    from psycopg2 import DatabaseError
+from django.db.utils import DatabaseError
 
-from django.shortcuts import redirect
+from django.utils.translation import ugettext as _
+from django.shortcuts import redirect, render
 from django.contrib.gis.geos import Point
-from django.contrib.gis.measure import D
+from django.contrib.gis.geos.geometry import GEOSGeometry
+from django.contrib.gis.db.models import Collect
+from django.views.decorators.csrf import csrf_exempt
+from django.db.models import FloatField
+from django.db.models.expressions import Func
+from django.contrib.gis.db.models.functions import Distance
+from django.contrib.gis.db.backends.postgis.adapter import PostGISAdapter
 
 from mapit.models import Postcode, Area, Generation
 from mapit.utils import is_valid_postcode, is_valid_partial_postcode
-from mapit.shortcuts import output_json, get_object_or_404, set_timeout, render
+from mapit.shortcuts import output_json, get_object_or_404, set_timeout
 from mapit.middleware import ViewException
 from mapit.ratelimitcache import ratelimit
 from mapit.views.areas import add_codes
@@ -38,7 +42,20 @@ enclosing_areas = {
 }
 
 
-@ratelimit(minutes=3, requests=100)
+class GeometryCentroidDistance(Func):
+    function = ''
+    arg_joiner = ' <-> '
+
+    def __init__(self, expression, geom, **extra):
+        if not isinstance(geom, GEOSGeometry):
+            raise TypeError("Please provide a geometry object.")
+        if not hasattr(geom, 'srid') or not geom.srid:
+            raise ValueError("Please provide a geometry attribute with a defined SRID.")
+        super(GeometryCentroidDistance, self).__init__(
+            expression, PostGISAdapter(geom), output_field=FloatField(), **extra)
+
+
+@ratelimit
 def postcode(request, postcode, format=None):
     if hasattr(countries, 'canonical_postcode'):
         canon_postcode = countries.canonical_postcode(postcode)
@@ -51,11 +68,11 @@ def postcode(request, postcode, format=None):
         raise ViewException(format, "Postcode '%s' is not valid." % postcode, 400)
     postcode = get_object_or_404(Postcode, format=format, postcode=postcode)
     try:
-        generation = int(request.REQUEST['generation'])
+        generation = int(request.GET['generation'])
     except:
         generation = Generation.objects.current()
     if not hasattr(countries, 'is_special_postcode') or not countries.is_special_postcode(postcode.postcode):
-        areas = add_codes(Area.objects.by_postcode(postcode, generation))
+        areas = list(add_codes(Area.objects.by_postcode(postcode, generation)))
     else:
         areas = []
 
@@ -86,7 +103,7 @@ def postcode(request, postcode, format=None):
         return render(request, 'mapit/postcode.html', {
             'postcode': postcode.as_dict(),
             'areas': areas,
-            'json_view': 'mapit.views.postcodes.postcode',
+            'json_view': 'mapit-postcode',
         })
 
     out = postcode.as_dict()
@@ -96,34 +113,33 @@ def postcode(request, postcode, format=None):
     return output_json(out)
 
 
-@ratelimit(minutes=3, requests=100)
-def partial_postcode(request, postcode, format='json'):
-    postcode = re.sub('\s+', '', postcode.upper())
+@ratelimit
+def partial_postcode(request, postcode, format=''):
+    postcode = re.sub(r'\s+', '', postcode.upper())
     if is_valid_postcode(postcode):
-        postcode = re.sub('\d[A-Z]{2}$', '', postcode)
+        postcode = re.sub(r'\d[A-Z]{2}$', '', postcode)
     if not is_valid_partial_postcode(postcode):
         raise ViewException(format, "Partial postcode '%s' is not valid." % postcode, 400)
-    try:
-        postcode = Postcode(
-            postcode=postcode,
-            location=Postcode.objects.filter(postcode__startswith=postcode).extra(
-                where=['length(postcode) = %d' % (len(postcode) + 3)]
-            ).collect().centroid
-        )
-    except:
+
+    location = Postcode.objects.filter(postcode__startswith=postcode).extra(
+        where=['length(postcode) = %d' % (len(postcode) + 3)]
+        ).aggregate(Collect('location'))['location__collect']
+    if not location:
         raise ViewException(format, 'Postcode not found', 404)
 
+    postcode = Postcode(postcode=postcode, location=location.centroid)
+
     if format == 'html':
         return render(request, 'mapit/postcode.html', {
             'postcode': postcode.as_dict(),
-            'json_view': 'mapit.views.postcodes.partial_postcode',
+            'json_view': 'mapit-postcode-partial',
         })
 
     return output_json(postcode.as_dict())
 
 
-@ratelimit(minutes=3, requests=100)
-def example_postcode_for_area(request, area_id, format='json'):
+@ratelimit
+def example_postcode_for_area(request, area_id, format=''):
     area = get_object_or_404(Area, format=format, id=area_id)
     try:
         pc = Postcode.objects.filter(areas=area).order_by()[0]
@@ -131,8 +147,6 @@ def example_postcode_for_area(request, area_id, format='json'):
         set_timeout(format)
         try:
             pc = Postcode.objects.filter_by_area(area).order_by()[0]
-        except QueryCanceledError:
-            raise ViewException(format, 'That query was taking too long to compute.', 500)
         except DatabaseError as e:
             if 'canceling statement due to statement timeout' not in e.args[0]:
                 raise
@@ -146,23 +160,39 @@ def example_postcode_for_area(request, area_id, format='json'):
     return output_json(pc)
 
 
+@csrf_exempt
 def form_submitted(request):
-    pc = request.POST.get('pc', None)
+    pc = request.POST.get('pc', '')
+    if hasattr(countries, 'canonical_postcode'):
+        pc = countries.canonical_postcode(pc)
     if not request.method == 'POST' or not pc:
         return redirect('/')
-    return redirect('mapit.views.postcodes.postcode', postcode=pc, format='html')
+    return redirect('mapit-postcode', postcode=pc, format='html')
 
 
-@ratelimit(minutes=3, requests=100)
-def nearest(request, srid, x, y, format='json'):
+@ratelimit
+def nearest(request, srid, x, y, format=''):
     location = Point(float(x), float(y), srid=int(srid))
     set_timeout(format)
+
     try:
-        postcode = Postcode.objects.filter(
-            location__distance_gte=(location, D(mi=0))).distance(location).order_by('distance')[0]
-    except QueryCanceledError:
-        raise ViewException(format, 'That query was taking too long to compute.', 500)
+        # Transform to database SRID for comparison (GeometryCentroidDistance does not yet do this)
+        location.transform(4326)
+    except:
+        raise ViewException(format, _('Point outside the area geometry'), 400)
+
+    try:
+        # Ordering will be in 'degrees', so fetch a few and sort by actual distance
+        postcodes = Postcode.objects.annotate(
+            centroid_distance=GeometryCentroidDistance('location', location)
+            ).annotate(
+                distance=Distance('location', location)
+            ).order_by('centroid_distance')[:100]
+        postcodes = sorted(postcodes, key=attrgetter('distance'))
+        postcode = postcodes[0]
     except DatabaseError as e:
+        if 'Cannot find SRID' in e.args[0]:
+            raise ViewException(format, e.args[0], 400)
         if 'canceling statement due to statement timeout' not in e.args[0]:
             raise
         raise ViewException(format, 'That query was taking too long to compute.', 500)
@@ -172,7 +202,7 @@ def nearest(request, srid, x, y, format='json'):
     if format == 'html':
         return render(request, 'mapit/postcode.html', {
             'postcode': postcode.as_dict(),
-            'json_view': 'mapit.views.postcodes.postcode',
+            'json_view': 'mapit-postcode',
         })
 
     pc = postcode.as_dict()
