@@ -1,12 +1,26 @@
 # This script is for a one off import of all the old ONS codes to a MapIt
 # containing only the new ones from a modern Boundary-Line.
 
+import codecs
 import csv
 import os.path
-from django.core.management.base import NoArgsCommand
+import sys
+from django.core.management.base import BaseCommand
 from mapit.models import Area, CodeType
 from psycopg2 import IntegrityError
 
+python_version = sys.version_info[0]
+
+
+def open_csv(filename):
+    if python_version < 3:
+        o = open(filename)
+    else:
+        o = codecs.open(filename, 'r', 'latin-1')
+    mapping = csv.reader(o)
+    next(mapping)
+    return mapping
+
 
 def process(new_code, old_code):
     try:
@@ -25,18 +39,14 @@ def process(new_code, old_code):
         raise Exception("Key already exists for %s, can't give it %s" % (area, old_code))
 
 
-class Command(NoArgsCommand):
+class Command(BaseCommand):
     help = 'Inserts the old ONS codes into mapit'
 
-    def handle_noargs(self, **options):
-        mapping = csv.reader(open(os.path.dirname(__file__) + '/../../data/BL-2010-10-code-change.csv'))
-        next(mapping)
-        for row in mapping:
-            new_code, name, old_code = row[0], row[1], row[3]
+    def handle(self, **options):
+        for row in open_csv(os.path.join(os.path.dirname(__file__), '../../data/BL-2010-10-code-change.csv')):
+            new_code, old_code = row[0], row[3]
             process(new_code, old_code)
 
-        mapping = csv.reader(open(os.path.dirname(__file__) + '/../../data/BL-2010-10-missing-codes.csv'))
-        next(mapping)
-        for row in mapping:
-            type, new_code, old_code, name = row
+        for row in open_csv(os.path.join(os.path.dirname(__file__), '../../data/BL-2010-10-missing-codes.csv')):
+            new_code, old_code = row[1], row[2]
             process(new_code, old_code)
